@@ -29,15 +29,15 @@ TextSummary <- function(text = clipr::read_clip(),
                         verbose = TRUE,
                         returnText = FALSE){
 
-  choices1 <- c("GPT-3.5", "GPT-4 (0613)", "Another LLM")
+  choices1 <- c("GPT-4o-mini (Default)", "GPT-4o", "GPT-4-turbo")
   selection1 <- utils::menu(choices1, title = "Which language model do you prefer?")
 
   if (selection1 == 1) {
-    Model = "gpt-3.5-turbo"
+    Model = "gpt-4o-mini"
   } else if (selection1 == 2) {
-    Model = "gpt-4-0613"
+    Model = "gpt-4o"
   } else if (selection1 == 3) {
-    return(message("No valid selection made."))
+    Model = "gpt-4-turbo"
   } else {
     return(message("No valid selection made."))
   }
@@ -133,9 +133,18 @@ TextSummary <- function(text = clipr::read_clip(),
 
     retry_count <- 0
     while (retry_count <= 2) {
-      res <- chat4R_history(history = history,
-                            Model = Model,
-                            temperature = temperature)
+      res_df <- chat4R_history(history = history,
+                              Model = Model,
+                              temperature = temperature)
+                              
+      # Extract content from data.frame
+      if (is.null(res_df) || !is.data.frame(res_df) || !"content" %in% names(res_df) || 
+          is.null(res_df$content) || length(res_df$content) == 0 || nchar(trimws(res_df$content)) == 0) {
+        stop("Invalid or empty response from chat4R_history", call. = FALSE)
+      }
+      
+      res <- as.character(res_df$content)
+      
       if(nchar(res) < Summary_block + 100){ break }
       retry_count <- retry_count + 1
     }
@@ -169,7 +178,7 @@ TextSummary <- function(text = clipr::read_clip(),
                             nch = 2000,
                             verbose = TRUE){
 
-  Model = "gpt-3.5-turbo"
+  Model = "gpt-4o-mini"
   Summary_block = nch*0.1
   temperature = 1
 
@@ -243,9 +252,18 @@ TextSummary <- function(text = clipr::read_clip(),
 
     retry_count <- 0
     while (retry_count <= 2) {
-      res <- chat4R_history(history = history,
-                            Model = Model,
-                            temperature = temperature)
+      res_df <- chat4R_history(history = history,
+                              Model = Model,
+                              temperature = temperature)
+                              
+      # Extract content from data.frame
+      if (is.null(res_df) || !is.data.frame(res_df) || !"content" %in% names(res_df) || 
+          is.null(res_df$content) || length(res_df$content) == 0 || nchar(trimws(res_df$content)) == 0) {
+        stop("Invalid or empty response from chat4R_history", call. = FALSE)
+      }
+      
+      res <- as.character(res_df$content)
+      
       if(nchar(res) < Summary_block + 100){ break }
       retry_count <- retry_count + 1
     }
