@@ -80,9 +80,17 @@ convertRscript2Function <- function(Model = "gpt-4o-mini",
   if(verbose){utils::setTxtProgressBar(pb, 2)}
 
   # Execute text generation
-  res <- chat4R_history(history = history,
-                        Model = Model,
-                        temperature = temperature)
+  res_df <- chat4R_history(history = history,
+                          Model = Model,
+                          temperature = temperature)
+
+  # Extract content from data.frame
+  if (is.null(res_df) || !is.data.frame(res_df) || !"content" %in% names(res_df) || 
+      is.null(res_df$content) || length(res_df$content) == 0 || nchar(trimws(res_df$content)) == 0) {
+    stop("Invalid or empty response from chat4R_history", call. = FALSE)
+  }
+  
+  res <- as.character(res_df$content)
 
   if(verbose){
     utils::setTxtProgressBar(pb, 3)
@@ -90,8 +98,8 @@ convertRscript2Function <- function(Model = "gpt-4o-mini",
 
   # Output the optimized code
   if(SelectedCode){
-    rstudioapi::insertText(text = as.character(res))
+    rstudioapi::insertText(text = res)
   } else {
-    return(clipr::write_clip(as.character(res)))
+    return(clipr::write_clip(res))
   }
 }
