@@ -79,16 +79,25 @@ createRcode <- function(Summary_nch = 100,
   if(verbose){utils::setTxtProgressBar(pb, 2)}
 
   # Execution
-  res <- chat4R_history(history=history,
-                        Model = Model,
-                        temperature = temperature)
+  res_df <- chat4R_history(history=history,
+                          Model = Model,
+                          temperature = temperature)
+
+  # Extract content from data.frame
+  if (is.null(res_df) || !is.data.frame(res_df) || !"content" %in% names(res_df) || 
+      is.null(res_df$content) || length(res_df$content) == 0 || nchar(trimws(res_df$content)) == 0) {
+    stop("Invalid or empty response from chat4R_history", call. = FALSE)
+  }
+  
+  res <- as.character(res_df$content)
+
   if(verbose){
     utils::setTxtProgressBar(pb, 3)
     cat("\n")}
 
   # Output
   if(SelectedCode){
-    rstudioapi::insertText(text = as.character(res))
+    rstudioapi::insertText(text = res)
   } else {
   if(verbose) {
     if(SlowTone) {
@@ -100,7 +109,7 @@ createRcode <- function(Summary_nch = 100,
     }
   }
 
-  return(clipr::write_clip(as.character(res)))
+  return(clipr::write_clip(res))
 
   }
 }
